<?php

namespace SMSGateway;

require_once './utils.php';

class SMSMasivos {
    // docs at: https://app.smsmasivos.com.mx/api-docs/v2#auth
    public static $auth_performed = false;
    public static $token = '';

    public static function sendSMS($gateway_fields, $mobile, $message, $test_call) {
        $username = $gateway_fields['api_key'];
        $country_code = $gateway_fields['country_code'];
        $token = self::_obtain_auth_credentials($api_key);

        if (!self::$auth_performed) return false;

        return self::process_sms($token, $country_code, $mobile, $message, $test_call);
    }

    public static function sendBulkSMS($gateway_fields, $messages, $test_call) {
        $username = $gateway_fields['api_key'];
        $country_code = $gateway_fields['country_code'];
        $token = self::_obtain_auth_credentials($api_key);

        if (!self::$auth_performed) return false;

        return self::process_sms($token, $country_code, $mobile, $message, $test_call);
    }

    public static function _obtain_auth_credentials($api_key) {
        if ($auth_performed) {
            return self::$token;
        }
        $curl = curl_init();

        $post_params = array(
            'api_key' => $api_key,
        );
        curl_setopt(
            $curl,
            CURLOPT_URL,
            'https://api.smsmasivos.com.mx/auth'
        );
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $post_params);
        $result = curl_exec($curl);
        $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $curl_error = curl_errno($curl);
        curl_close($curl);

        if ($curl_error !== 0) {
            return false;
        }

        $is_success = 200 <= $code && $code < 300;

        if ($is_success) {
            $auth_credentials_decoded = json_decode($result);
            self::$token = $auth_credentials_decoded['token'];
            return self::$token;
        }

        return '';
    }

    public static function process_sms($token, $country_code, $messages, $test_call) {
        $curl = curl_init();
        $chunked_messages = array_chunk($messages, self::$chunks);
        $failed_sent = [];
        $results = [];
        $fixed_message = '';

        foreach($chunked_messages as $message_batch) {
            $mobiles = [];

            foreach($message_batch as $id => $message_descriptor) {
                foreach($message_descriptor as $mobile => $message) {
                    $fixed_message = $message;
                    $mobiles[] = $mobile;
                }
            }

            $post_params = array(
                'numbers' => join(',', $mobiles),
                'message' => $fixed_message,
                'country_code' => $country_code,
            );
            curl_setopt($curl, CURLOPT_URL, 'https://api.smsmasivos.com.mx/sms/send');
            curl_setopt(
                $curl,
                CURLOPT_HTTPHEADER,
                array(
                    'token: ' . $token,
                )
            );
            curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $post_params);

            $result = curl_exec($curl);
            $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $curl_error = curl_errno($curl);
            curl_close($curl);

            if($test_call) {
                $results[] = $result;
            }

            $is_success = 200 <= $code && $code < 300;

            if ($is_success && $curl_error !== 0) {
            } else {
                $failed_sent += $mobiles;
            }
        }

        if($test_call) return $results;

        return \last_sent_from_failed($messages, $failed_sent);
    }
}
