<?php

/*
 * Plugin Name: DIGITS: Additional Fields & Logic Builder Addon
 * Description: Enhance the capabilities of your form by additional and advanced fields, or build a logic to hide/show some field when that particular logic is met.
 * Version: 1.5.7
 * Plugin URI: https://digits.unitedover.com/addons
 * Author URI: https://www.unitedover.com/
 * Author: UnitedOver
 * Text Domain: digadvlogic
 * Requires PHP: 5.5
 * Domain Path: /languages
 */

if (!defined('ABSPATH')) {
    exit;
}

function digadv_version()
{
    return '1.5.7';
}


require dirname(__FILE__) . '/Puc/v4p6/Factory.php';
require dirname(__FILE__) . '/Puc/v4/Factory.php';
require dirname(__FILE__) . '/Puc/v4p6/Autoloader.php';
new Puc_v4p6_Autoloader();

foreach (
    array(
        'Plugin_UpdateChecker' => 'Puc_v4p6_Plugin_UpdateChecker',
        'Vcs_PluginUpdateChecker' => 'Puc_v4p6_Vcs_PluginUpdateChecker',
    )
    as $pucGeneralClass => $pucVersionedClass
) {
    Puc_v4_Factory::addVersion($pucGeneralClass, $pucVersionedClass, '4.6');

    Puc_v4p6_Factory::addVersion($pucGeneralClass, $pucVersionedClass, '4.6');
}


function digits_addon_digadvlogic()
{
    return 'customfields';
}


$digadvlogicUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
    'https://bridge.unitedover.com/updates/changelog/addons.php?addon=digadvlogic',
    __FILE__,
    'digadvlogic'
);

$digadvlogicUpdateChecker->addQueryArgFilter('digadvlogic_filter_update_checks');
function digadvlogic_filter_update_checks($queryArgs)
{


    $queryArgs['license_key'] = get_site_option('dig_purchasecode');

    $queryArgs['request_site'] = network_home_url();

    $queryArgs['license_type'] = get_site_option('dig_license_type', 1);

    $plugin_data = get_plugin_data(__FILE__);
    $plugin_version = $plugin_data['Version'];

    $queryArgs['version'] = $plugin_version;


    return $queryArgs;
}


function digadvlogic_addon($list)
{
    $list[] = 'digadvlogic';
    return $list;
}

add_filter('digits_addon', 'digadvlogic_addon');


function dig_ext_setup_wizard()
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) return;


    if (wp_script_is('dig-ext-script')) {
        return;
    }
    wp_register_style('dig-ext-style', plugins_url('/css/style.css', __FILE__), array(), digadv_version(), 'all');
    wp_register_script('dig-ext-script', plugins_url('/js/script.js', __FILE__), array('digits-login-script'), digadv_version(), true);


    $jsData = array(
        'ajax_url' => admin_url('admin-ajax.php'),
        'cross' => plugins_url('/images/cross.png', __FILE__),
        "ohsnap" => __("Oh Snap!", "digits"),
        'face' => plugins_url('/images/face.png', __FILE__),
        'uploadAllFiles' => __('Please upload all the required files.', 'digadvlogic'),
        'minReqFiles' => __('{{field}} requires minimum of {{no}} files to be uploaded.', 'digadvlogic'),
        'fileUploaded' => __('File is being uploaded.', 'digadvlogic'),
        'delFinal' => __('Do you really want to delete {{file}}. This action can\'t be reversed.', 'digadvlogic'),
        "Email" => __("Email", "digits"),
        "Mobileno" => __("Mobile Number", "digits"),
        'EmailMobNo' => __('Email/Mobile', 'digits'),

    );

    wp_localize_script('dig-ext-script', 'dig_ext_script', $jsData);

    wp_enqueue_style('dig-ext-style');
    wp_enqueue_script('dig-ext-script');
}

add_action('admin_init', 'dig_ext_setup_wizard');
add_action('login_enqueue_scripts', 'dig_ext_setup_wizard');

function dig_adv_update_fields($fields)
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) return $digpc;
    $custom_field = dig_adv_custom_fields();
    return array_merge($fields, $custom_field);
}

add_filter('dig_all_custom_fields', 'dig_adv_update_fields');
function dig_adv_custom_fields()
{
    return array(
        'time' => array('name' => 'Time', 'force_required' => 0, 'meta_key' => 1, 'options' => 0, 'slug' => 'time'),
        'dateandtime' => array('name' => 'Date & Time', 'force_required' => 0, 'meta_key' => 1, 'options' => 0, 'slug' => 'dateandtime'),
        'daterange' => array('name' => 'Date Range', 'force_required' => 0, 'meta_key' => 1, 'options' => 0, 'slug' => 'daterange'),
        'dateandtimerange' => array('name' => 'Date & Time Range', 'force_required' => 0, 'meta_key' => 1, 'options' => 0, 'slug' => 'dateandtimerange'),
        'heading' => array('name' => 'Heading', 'force_required' => 1, 'meta_key' => 0, 'options' => 0, 'slug' => 'heading'),
        'html' => array('name' => 'HTML', 'force_required' => 1, 'meta_key' => 0, 'options' => 0, 'slug' => 'html'),
        'upload' => array('name' => 'Upload', 'force_required' => 0, 'meta_key' => 1, 'options' => 0, 'slug' => 'upload'),
    );
}


function dig_ext_custom_fields_list()
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) return;


    $custom_field = dig_adv_custom_fields();

    foreach ($custom_field as $fieldname => $type) {
        if (isset($type['hidden']) && $type['hidden'] == 1) continue; ?>
        <div class="dig_sb_field_types dig_sb_field_list" id="dig_cust_list_type_<?php echo $fieldname; ?>"
             data-val='<?php echo $fieldname; ?>' data-configure_fields='<?php echo json_encode($type); ?>'>
            <?php _e($type['name'], 'digadvlogic'); ?>
        </div>
    <?php }
}

add_action("dig_custom_fields_list", "dig_ext_custom_fields_list");


add_action('dig_custom_fields_options', 'dig_add_adv_fields_data');
function dig_add_adv_fields_data()
{

    ?>
    <div class="dig_sb_field dig_sb_field_html dig_sb_extr_fields" data-req="1">
        <div class="dig_sb_field_label">
            <label for="dig_csf_add_html"><?php _e('HTML'); ?><span class="dig_sb_required">*</span></label>
        </div>
        <div class="dig_sb_field_input">
            <textarea type="text" id="dig_csf_add_html" name="adv_html" rows="3"></textarea>
        </div>
    </div>


    <div class="dig_sb_field dig_sb_field_upload dig_sb_extr_fields" data-req="1">
        <div class="dig_sb_field_label">
            <label for="dig_csf_max_upload_size"><?php _e('Maximum Upload Size (in MB)'); ?><span
                        class="dig_sb_required">*</span></label>
        </div>
        <div class="dig_sb_field_input">
            <input type="number" id="dig_csf_max_upload_size" name="max_size"/>
        </div>
    </div>
    <div class="dig_sb_field dig_sb_field_upload dig_sb_extr_fields">
        <div class="dig_sb_field_label">
            <label for="dig_csf_max_files"><?php _e('Maximum no of files that can be uploaded'); ?></label>
        </div>
        <div class="dig_sb_field_input">
            <input type="number" id="dig_csf_max_files" name="max_files"/>
        </div>
    </div>
    <div class="dig_sb_field dig_sb_field_upload dig_sb_extr_fields">
        <div class="dig_sb_field_label">
            <label for="dig_csf_min_files"><?php _e('Minimum no of files that can be uploaded'); ?></label>
        </div>
        <div class="dig_sb_field_input">
            <input type="number" id="dig_csf_min_files" name="min_files"/>
        </div>
    </div>

    <div class="dig_sb_field dig_sb_field_upload dig_sb_extr_fields">
        <div class="dig_sb_field_label">
            <label for="dig_csf_allw_ext"><?php _e('Allowed Extensions'); ?></label>
        </div>
        <div class="dig_sb_field_input">
            <input type="text" id="dig_csf_allw_ext" name="allw_ext"/>
        </div>
    </div>


    <?php
}


function dig_html_hide_field($hide, $type, $meta_key)
{

    if ($type == 'html') {
        $hide = true;
    }
    return $hide;
}

add_filter('dig_show_field_to_loggedin_user', 'dig_html_hide_field', 10, 3);


add_action("wp_ajax_nopriv_dig_upload_file", "dig_upload_file");
add_action("wp_ajax_dig_upload_file", "dig_upload_file");
function dig_upload_file()
{

    $nounce = $_POST['dig_filenonce'];
    if (!wp_verify_nonce($nounce, 'dig_filenonce')) {
        return;
    }


    if (!empty($_FILES['file'])) {

        $field_name = $_POST['field_name'];
        $cf = digadv_get_field($field_name);


        if ($cf == null) {
            wp_send_json_error(array('message' => 'Error'));
            die();
        }
        $allowed_extensions = explode(",", $cf['allw_ext']);
        $max_size = $cf['max_size'] * 1024 * 1024;
        $uploadedfile = $_FILES['file'];


        $path_parts = pathinfo($uploadedfile["name"]);

        $ext = $path_parts['extension'];
        $file_name = $path_parts['basename'];

        if ($uploadedfile['size'] > $max_size) {
            wp_send_json_error(array('message' => 'Sorry, file is too large.'));
            die();
        } else if ((!empty($allowed_extensions) && !in_array($ext, $uploadedfile)) || $ext == 'php') {
            wp_send_json_error(array('message' => 'Sorry, this type of file is not allowed.'));
            die();
        }


        $upload_overrides = array('test_form' => false);
        add_filter('upload_dir', 'digits_file_upload_tmp_dir');
        add_filter('wp_unique_filename', 'digits_unq_file_name', 10, 4);

        $movefile = wp_handle_upload($uploadedfile, $upload_overrides);
        remove_filter('upload_dir', 'digits_file_upload_tmp_dir');
        remove_filter('wp_unique_filename', 'digits_unq_file_name');


        if (isset($movefile['error'])) {
            wp_send_json_error(array('message' => $movefile['error']));
            die();
        }
        if ($movefile) {

            $wp_upload_dir = wp_get_upload_dir();
            $base_dir = $wp_upload_dir['basedir'] . '/digits/';
            $tmp_base_dir = $base_dir . 'tmp/';

            $fid = str_replace($tmp_base_dir, '', $movefile['file']);

            $type = $movefile['type'];

            wp_send_json_success(array("fname" => $file_name, "fid" => $fid, "nonce" => wp_create_nonce($fid), "type" => $type));
        } else {
            wp_send_json_error(array('message' => 'Error'));
        }

    }


    die();
}

add_action("wp_ajax_nopriv_dig_delete_file", "dig_delete_file");
add_action("wp_ajax_dig_delete_file", "dig_delete_file");
function dig_delete_file()
{
    $nounce = $_POST['nonce'];
    if (!wp_verify_nonce($nounce, $_POST['fid'])) {
        die();
    }
    $wp_upload_dir = wp_get_upload_dir();

    $fname = $_POST['fname'];
    $fid = $_POST['fid'];

    if (is_user_logged_in()) {
        if (!current_user_can('edit_user')) {
            $user_id = get_current_user_id();
        } else {
            $user_id = sanitize_text_field($_POST['user_id']);
        }

        wp_delete_file($wp_upload_dir['basedir'] . '/digits/userfiles/' . md5($user_id) . '/' . $fid);

        $field_name = $_POST['field_name'];
        $cf = digadv_get_field($field_name);
        $meta_key = sanitize_text_field($cf['meta_key']);


        $e_value = json_decode(get_user_meta($user_id, $meta_key, true), true);

        unset($e_value[$fid]);

        update_user_meta($user_id, $meta_key, json_encode($e_value));

    } else {
        wp_delete_file($wp_upload_dir['basedir'] . '/digits/tmp/' . $fid);
    }
}

function digadv_get_field($field_name)
{
    $reg_custom_fields = stripslashes(base64_decode(get_option("dig_reg_custom_field_data", "e30=")));
    $reg_custom_fields = json_decode($reg_custom_fields, true);

    if (array_key_exists($field_name, $reg_custom_fields)) {
        return $reg_custom_fields[$field_name];
    }

    foreach ($reg_custom_fields as $key => $field) {
        if ($field['meta_key'] == $field_name) {
            return $field;
        }
    }
    return false;
}

function digits_file_upload_tmp_dir($dirs)
{
    $dirs['subdir'] = '/digits/tmp';
    $dirs['path'] = $dirs['basedir'] . '/digits/tmp';
    $dirs['url'] = $dirs['baseurl'] . '/digits/tmp';

    return $dirs;
}


function dig_ext_show_fields($con, $custom_class, $values, $type, $e_value, $meta_key, $user_id, $login_page)
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) return false;


    if (!current_user_can('edit_user')) {
        $user_id = 0;
    }

    if ($type) {
        wp_register_style('dig-ext-fileupload-style', plugins_url('/css/fileupload.css', __FILE__), "dig-ext-style", digadv_version(), 'all');
        wp_register_script('dig-ext-fileupload-script', plugins_url('/js/bootstrap-fileupload.min.js', __FILE__), "dig-ext-script", digadv_version(), true);
        wp_enqueue_style('dig-ext-fileupload-style');
        wp_enqueue_script('dig-ext-fileupload-script');
        dig_ext_setup_wizard();
    }

    if ($type == "upload") {

        $wp_upload_dir = wp_get_upload_dir();


        $base_dir = $wp_upload_dir['basedir'] . '/digits/';


        $upl_base_dir = $base_dir . 'userfiles/' . md5($user_id) . '/';


        ?>
        <input type="hidden" class="current_user_id" value="<?php echo $user_id; ?>"/>


        <div class="fileupload fileupload-new" data-provides="fileupload">
            <div class="input-append <?php if ($login_page != 2) {
                echo 'dig_pg_border_box';
            } ?>">

                <?php
                if ($login_page == 2) {
                    ?>
                    <div class="dig_adv_read_only_inp"><input type="text" readonly <?php echo $custom_class; ?>></div>
                    <?php
                }
                ?>

                <span class="btn btn-file">

                    <span class="btn_select_file">
                    <span class="fileupload-new">Select file</span><span class="fileupload-exists">Select File</span>
                        <span class="btn_select_file_btn_bg"></span>
                    </span>
                    <input type="file" <?php if ($values['required'] == 1) echo "dig-req='required'"; ?> f-name="<?php echo esc_attr($values['meta_key']); ?>"
                           max_size="<?php echo $values['max_size']; ?>" <?php if ($values['max_files'] > 1) echo 'multiple limit="' . $values['max_files'] . '"'; ?> <?php echo 'min="' . $values['min_files'] . '"'; ?> accept="<?php echo $values['allw_ext']; ?>"
                           id="digits_reg_<?php echo $meta_key; ?>">

                </span>

                <input type="hidden" class="dig_uploaded_file_data" name="digits_reg_<?php echo $meta_key; ?>"
                       value="<?php echo esc_attr(stripslashes($e_value)); ?>"/>

                <input type="hidden" name="dig_filenonce" class="dig_filenonce"
                       value="<?php echo wp_create_nonce('dig_filenonce'); ?>">

            </div>


            <table class="file_upload_status">
                <?php

                if (!empty($e_value)) {
                    $e_value = json_decode($e_value, true);
                    foreach ($e_value as $fid => $details) {
                        $fid = esc_attr($fid);
                        $name = esc_attr($details['name']);
                        $type = esc_attr($details['type']);
                        $nonce = wp_create_nonce($fid);

                        if (!file_exists($upl_base_dir . $fid)) continue;
                        //'id':msg.data.fid,'nce':msg.data.nonce,'fname'
                        ?>
                        <tr id="<?php echo $fid; ?>" fname="<?php echo $name; ?>" nce="<?php echo $nonce; ?>"
                            type=""<?php echo $type; ?>>
                            <td>
                                <div class="file_contents">
                                    <div class="file_name">
                                        <a href="<?php echo create_dig_download_filelink($nonce, $fid, $name, $type, $user_id); ?>"
                                           download="<?php echo $name; ?>"><?php echo $name; ?></a>
                                    </div>
                                    <div class="progress dig_sbtncolor"></div>
                                </div>
                            </td>
                            <td class="delete"><img class="close_condition dig_delete_file_upload"
                                                    src="<?php echo plugins_url('/images/cross.png', __FILE__); ?>"/>
                            </td>
                        </tr>
                        <?php
                    }
                }
                ?>
            </table>


        </div>

        <script>
            var form = jQuery('.fileupload').closest("form");
            jQuery(form).attr("enctype", "multipart/form-data");
        </script>
        <?php return true;
    } else if ($type == 'heading') {

        echo '<h2>' . $values['label'] . '</h2>';
        return true;
    } else if ($type == 'html') {
        echo str_replace(array('/x22', '/x27'), array('"', "'"), $values['adv_html']);
        return true;
    } else if (in_array($type, array('dateandtime', 'dateandtimerange', 'daterange'))) {
        $e_range = '';
        if (in_array($type, array('dateandtimerange', 'daterange'))) {
            $e_range = 'data-range="true"';
        }


        ?>
        <div class="digits-input-wrapper">
            <input type="text"
                   name="digits_reg_<?php echo $meta_key; ?>"
                   id="digits_reg_<?php echo $meta_key; ?>" <?php echo $custom_class; ?> <?php if ($values['required'] == 1) echo "required"; ?>
                   value="<?php if ($e_value) echo $e_value; ?>" <?php echo $e_range; ?> />
        </div>
        <?php
        return true;
    }
    return $con;
}

add_filter("before_dig_show_fields", "dig_ext_show_fields", 10, 8);


function create_dig_download_filelink($nonce, $fid, $fname, $type, $user_id)
{
    $base_url = get_site_url();

    $url_parts = parse_url($base_url);
    parse_str($url_parts['query'], $params);

    $params['type'] = 'download';
    $params['fid'] = $fid;
    $params['nonce'] = $nonce;
    $params['fname'] = $fname;
    $params['file_type'] = $type;
    $params['user_id'] = $user_id;


    $url_parts['query'] = http_build_query($params);

    return $url_parts['scheme'] . '://' . $url_parts['host'] . $url_parts['path'] . '?' . $url_parts['query'];
}

function dig_download_file()
{
    if (isset($_GET['type']) && isset($_GET['nonce'])) {

        if ($_GET['type'] != 'download') return;

        if (!is_user_logged_in()) return;

        $nounce = $_GET['nonce'];
        if (!wp_verify_nonce($nounce, $_GET['fid'])) {
            return;
        }
        $wp_upload_dir = wp_get_upload_dir();

        $fname = $_GET['fname'];
        $fid = $_GET['fid'];


        if (!current_user_can('edit_user')) {
            $user_id = get_current_user_id();
        } else {
            $user_id = sanitize_text_field($_GET['user_id']);
        }


        $base_dir = $wp_upload_dir['basedir'] . '/digits/';
        $tmp_base_dir = $base_dir . 'tmp/';
        $upl_base_dir = $base_dir . 'userfiles/' . md5($user_id) . '/';


        $attachment_location = $upl_base_dir . $fid;
        if (file_exists($attachment_location)) {

            header($_SERVER["SERVER_PROTOCOL"] . " 200 OK");
            header("Cache-Control: public"); // needed for internet explorer
            header("Content-Type: " . $_GET['file_type']);
            header("Content-Transfer-Encoding: Binary");
            header("Content-Length:" . filesize($attachment_location));
            header("Content-Disposition: attachment; filename=" . $fname);
            readfile($attachment_location);
            die();
        } else {
            return;
        }


    }

}

add_action('init', 'dig_download_file', 10);


function dig_ext_show_label($type)
{
    if ($type == "upload")
        return "id='fileupload_label'";
    else
        return "";
}

add_filter("before_dig_custom_show_label", "dig_ext_show_label", 10, 1);

function dig_ext_show_line($type)
{
    if ($type == "upload")
        return "id='fileupload_span'";
    else
        return "";
}

add_filter("before_dig_custom_show_line", "dig_ext_show_line", 10, 1);

add_action('dig_field_type_time', 'dig_field_add_date_script');
add_action('dig_field_type_daterange', 'dig_field_add_date_script');
add_action('dig_field_type_dateandtime', 'dig_field_add_date_script');
add_action('dig_field_type_dateandtimerange', 'dig_field_add_date_script');


function dig_adv_validate_fields($continue, $values)
{

    if ($continue) return $continue;
    $reg_lb = json_decode(stripslashes(get_option("dig_reg_lb_data")), true);

    $key = cust_dig_filter_string($values['meta_key']);
    foreach ($reg_lb as $conditions) {
        foreach ($conditions['then'] as $condition) {

            $action = $condition['action'];
            if ($action == 'hide_fields' || $action == 'show_fields') {
                $field = preg_replace('/dig_cs_/', '', $condition['value'], 1);
                if ($field == $key) {
                    $continue = true;
                    break;
                }
            }


        }
        if ($continue) break;
    }
    return $continue;
}

add_filter('dig_require_field', 'dig_adv_validate_fields', 10, 2);


function dig_ext_validate_reg_field($e_value, $type, $values, $post_index)
{


    $e_value = $_POST[$post_index];
    if ($type == "upload") {
        if (!empty($e_value)) {
            $required = $values['required'];


            $e_json = json_decode(stripslashes($e_value));

            $size = count(get_object_vars($e_json));
            $error = new WP_Error();

            if ($values['max_files'] > 0) {
                if ($size > $values['max_files']) {
                    $error->add("limit", __('You are only allowed to upload a maximum of ' . $values['max_files'] . ' files!', 'digadvlogic'));

                    return $error;
                }
            }
            if ($values['min_files'] > 0) {
                if ($size < $values['min_files']) {
                    $error->add("limit", __('You are only allowed to upload a minimum of ' . $values['min_files'] . ' files!', 'digadvlogic'));

                    return $error;
                }
            }
            if ($required == 1 && empty($e_json)) {
                return null;
            }

            return $e_value;
        } else {
            return null;
        }
    } else
        return null;
}

add_filter("dig_validate_reg_field", "dig_ext_validate_reg_field", 10, 4);


function dig_ext_update_reg_field($e_value, $type, $values, $user_id)
{

    if ($type == "upload") {
        if (!empty($e_value)) {


            $e_json = json_decode(stripslashes($e_value));

            $wp_upload_dir = wp_get_upload_dir();
            $base_dir = $wp_upload_dir['basedir'] . '/digits/';
            $tmp_base_dir = $base_dir . 'tmp/';
            $upl_base_dir = $base_dir . 'userfiles/' . md5($user_id) . '/';

            if (!file_exists($upl_base_dir)) {
                wp_mkdir_p($upl_base_dir);
            }
            foreach ($e_json as $key => $e) {
                rename($tmp_base_dir . $key, $upl_base_dir . $key);

            }

            return $e_value;
        } else {
            return null;
        }
    } else
        return $e_value;
}

add_filter("dig_update_reg_field", "dig_ext_update_reg_field", 10, 5);


function dig_adv_delete_files($user_id)
{
    $wp_upload_dir = wp_get_upload_dir();
    $base_dir = $wp_upload_dir['basedir'] . '/digits/';
    $upl_base_dir = $base_dir . 'userfiles/' . md5($user_id) . '/';

    dig_deleteDirectory($upl_base_dir);

}

add_action('delete_user', 'dig_adv_delete_files');

function dig_deleteDirectory($dir)
{
    if (!file_exists($dir)) {
        return true;
    }

    if (!is_dir($dir)) {
        return unlink($dir);
    }

    foreach (scandir($dir) as $item) {
        if ($item == '.' || $item == '..') {
            continue;
        }

        if (!dig_deleteDirectory($dir . DIRECTORY_SEPARATOR . $item)) {
            return false;
        }

    }

    return rmdir($dir);
}


function digits_unq_file_name($filename2, $ext, $dir, $unique_filename_callback)
{
    // make filter magic happen here...
    $n_name = md5(rand() . time());

    return $n_name;
}

;

function digits_file_upload_dir($dirs)
{
    $dirs['subdir'] = '/digits/uploads';
    $dirs['path'] = $dirs['basedir'] . '/digits/uploads';
    $dirs['url'] = $dirs['baseurl'] . '/digits/uploads';

    return $dirs;
}


function dig_ext_require_field($continue, $values)
{
    if (in_array(strtolower($values['type']), array("html", "heading")))
        return true;
    else
        return $continue;
}

add_filter("dig_require_field", "dig_ext_require_field", 10, 2);

function dig_ext_custom_section($default_reg_fields, $reg_custom_fields)
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) return;


    dig_field_add_date_script();


    $reg_lb = stripslashes(get_option("dig_reg_lb_data"));

    if (!$reg_lb) $reg_lb = "{}"; ?>
    <input type="hidden" id="dig_reg_lb" name="dig_reg_lb"
           value='<?php echo $reg_lb; ?>'/>

    <div class="dig_reg_lb">
        <div style="font-size: 22px;color: #1B2A3C;font-style: italic;  padding-bottom: 40px;padding-top: 20px;">
            REGISTRATION FORM LOGIC BUILDER
        </div>
        <div class="dig_reg_lb_body">
            <table style="width: 100%;">
                <tr>
                    <td class="dig_small_header">CONDITIONS</td>
                    <td class="dig_small_header">ACTIONS</td>
                </tr>

                <?php
                $reg_lb = json_decode($reg_lb, true);
                $row_count = 0;
                foreach ($reg_lb as $index1 => $row) {
                    $if = $row['if'];
                    $then = $row['then'];
                    ?>
                    <tr class="dig_lb_row" for="<?php echo $row_count; ?>">
                        <td class="condition_section">
                            <table style="width: 100%; border-collapse: collapse;">
                                <?php
                                $condition_count = 0;
                                foreach ($if as $index2 => $condition) { ?>
                                    <?php if ($condition_count > 0) { ?>
                                        <tr class="dig_lb_logic" index="<?php echo $condition_count; ?>">

                                            <td></td>
                                            <td colspan="3" style="text-align: center;">
                                                <?php if ($condition_count > 0) {
                                                    echo $condition['logic'] == 'and' ? '<span class="logic_btn selected_logic">AND</span>' : '<span class="logic_btn">AND</span>';
                                                    echo $condition['logic'] == 'or' ? '<span class="logic_btn selected_logic">OR</span>' : '<span class="logic_btn">OR</span>';
                                                } ?>
                                            </td>
                                        </tr>
                                    <?php } ?>
                                    <tr class="dig_lb_condition" index="<?php echo $condition_count; ?>">
                                        <td style="font-weight: bold; vertical-align: middle;"><?php if ($condition_count == 0) echo "IF"; ?></td>
                                        <td>
                                            <select class="dig_lb_select">
                                                <option value="" selected="" data-display="(field)">Nothing</option>
                                                <?php foreach ($default_reg_fields as $reg_field => $values) {
                                                    $value = "dig_cs_" . cust_dig_filter_string($values['id']); ?>
                                                    <option <?php if ($value == $condition['field']) echo "selected"; ?>
                                                            value="<?php echo $value; ?>"><?php echo $values['name']; ?></option>
                                                <?php } ?>
                                                <?php foreach ($reg_custom_fields as $reg_field => $values) {
                                                    $value = "dig_cs_" . cust_dig_filter_string($values['meta_key']); ?>
                                                    <option <?php if ($value == $condition['field']) echo "selected"; ?>
                                                            value="<?php echo $value; ?>"><?php echo $values['label']; ?></option>
                                                <?php } ?>
                                            </select>
                                        </td>
                                        <td>
                                            <select class="dig_lb_trigger">
                                                <option value="" selected="" data-display="(trigger)">Nothing</option>
                                                <option value="is_equal_to" <?php if ($condition['trigger'] == "is_equal_to") echo "selected"; ?>>
                                                    is equal to
                                                </option>
                                                <option value="is_not_equal_to" <?php if ($condition['trigger'] == "is_not_equal_to") echo "selected"; ?>>
                                                    is not equal to
                                                </option>
                                                <option value="contains" <?php if ($condition['trigger'] == "contains") echo "selected"; ?>>
                                                    contains
                                                </option>
                                                <option value="does_not_contain" <?php if ($condition['trigger'] == "does_not_contain") echo "selected"; ?>>
                                                    does not contains
                                                </option>
                                                <option value="is_greater_than" <?php if ($condition['trigger'] == "is_greater_than") echo "selected"; ?>>
                                                    is greater than
                                                </option>
                                                <option value="is_less_than" <?php if ($condition['trigger'] == "is_less_than") echo "selected"; ?>>
                                                    is less than
                                                </option>
                                            </select>
                                        </td>
                                        <td>
                                            <input type="text" value="<?php echo $condition['value']; ?>"
                                                   class="dig_lb_value">
                                        </td>
                                        <td style="vertical-align: middle;">
                                            <img class="close_condition" for="<?php echo $condition_count; ?>"
                                                 src="<?php echo content_url('/plugins/digits/assets/images/cross.png', __FILE__); ?>">
                                        </td>
                                    </tr>
                                    <?php
                                    $condition_count++;
                                } ?>
                                <input type="hidden" value="<?php echo $condition_count; ?>" class="condition_count">
                                <tr>
                                    <td></td>
                                    <td colspan="3">
                                        <div class="add_condition_row">+ ADD CONDITION ROW</div>
                                    </td>
                                </tr>
                            </table>
                        </td>
                        <td class="result_section">
                            <table style="width: 100%; border-collapse: collapse;">
                                <?php
                                $result_count = 0;
                                foreach ($then as $index2 => $result) { ?>
                                    <tr class="dig_lb_result" index="<?php echo $result_count; ?>">
                                        <td style="font-weight: bold; vertical-align: middle;"><?php if ($result_count == 0) echo "THEN"; ?></td>
                                        <td>
                                            <select class="dig_lb_action">
                                                <option value="" selected="" data-display="(action)">Nothing</option>
                                                <option value="show_fields" <?php if ($result['action'] == "show_fields") echo "selected"; ?>>
                                                    show fields
                                                </option>
                                                <option value="hide_fields" <?php if ($result['action'] == "hide_fields") echo "selected"; ?>>
                                                    hide fields
                                                </option>
                                                <option value="set_value_of_field" <?php if ($result['action'] == "set_value_of_field") echo "selected"; ?>>
                                                    set value of field
                                                </option>
                                                <option value="change_value_of_field" <?php if ($result['action'] == "change_value_of_field") echo "selected"; ?>>
                                                    change value of field
                                                </option>
                                                <option value="redirect_to" <?php if ($result['action'] == "redirect_to") echo "selected"; ?>>
                                                    redirect to
                                                </option>
                                            </select>
                                        </td>
                                        <?php if ($result['action'] == 'show_fields' || $result['action'] == 'hide_fields') { ?>
                                            <td colspan="2" class="dig_lb_result_section">
                                                <select class="dig_lb_result_type1">
                                                    <?php foreach ($default_reg_fields as $reg_field => $values) {
                                                        $value = "dig_cs_" . cust_dig_filter_string($values['id']); ?>
                                                        <option <?php if ($value == $result['value']) echo "selected"; ?>
                                                                value="<?php echo $value; ?>"><?php echo $values['name']; ?></option>
                                                    <?php } ?>
                                                    <?php foreach ($reg_custom_fields as $reg_field => $values) {
                                                        $value = "dig_cs_" . cust_dig_filter_string($values['meta_key']); ?>
                                                        <option <?php if ($value == $result['value']) echo "selected"; ?>
                                                                value="<?php echo $value; ?>"><?php echo $values['label']; ?></option>
                                                    <?php } ?>
                                                </select>
                                            </td>
                                        <?php } else if ($result['action'] == 'redirect_to') { ?>
                                            <td colspan="2" class="dig_lb_result_section">
                                                <input type="text" value="<?php echo $result['value']; ?>"
                                                       class="dig_lb_result_type3">
                                            </td>
                                        <?php } else {
                                            $result_field = isset($result['field']) ? $result['field'] : '';
                                            ?>
                                            <td class="dig_lb_result_section">
                                                <select class="dig_lb_result_type2_field">
                                                    <option value="" selected="" data-display="(field)">Nothing</option>
                                                    <?php foreach ($default_reg_fields as $reg_field => $values) {
                                                        $value = "dig_cs_" . cust_dig_filter_string($values['id']); ?>
                                                        <option <?php if ($value == $result_field) echo "selected"; ?>
                                                                value="<?php echo $value; ?>"><?php echo $values['name']; ?></option>
                                                    <?php } ?>
                                                    <?php foreach ($reg_custom_fields as $reg_field => $values) {
                                                        $value = "dig_cs_" . cust_dig_filter_string($values['meta_key']); ?>
                                                        <option <?php if ($value == $result_field) echo "selected"; ?>
                                                                value="<?php echo $value; ?>"><?php echo $values['label']; ?></option>
                                                    <?php } ?>
                                                </select>
                                            </td>
                                            <td class="dig_lb_result_section">
                                                <input type="text" value="<?php echo $result['value']; ?>"
                                                       class="dig_lb_result_type2_value">
                                            </td>
                                        <?php } ?>
                                        <td style="vertical-align: middle;">
                                            <img class="close_result" for="<?php echo $result_count; ?>"
                                                 src="<?php echo content_url('/plugins/digits/assets/images/cross.png', __FILE__); ?>">
                                        </td>
                                        <td style="vertical-align: middle;" class="close_row">
                                            <?php if ($result_count == 0) { ?>
                                                <img for="<?php echo $row_count; ?>"
                                                     src="<?php echo content_url('/plugins/digits/assets/images/cross.png', __FILE__); ?>">
                                            <?php } ?>
                                        </td>
                                    </tr>
                                    <?php $result_count++;
                                } ?>
                                <input type="hidden" value="<?php echo $result_count; ?>" class="result_count">
                                <tr>
                                    <td></td>
                                    <td colspan="4">
                                        <div class="add_action_row">+ ADD ACTION ROW</div>
                                    </td>
                                    <td></td>
                                </tr>
                            </table>
                        </td>
                    </tr>
                    <?php
                    $row_count++;
                } ?>
                <tr>
                    <td colspan="2">
                        <div class="add_logic">+ ADD LOGIC</div>
                    </td>
                </tr>
                <input type="hidden" value="<?php echo $row_count; ?>" class="row_count">
            </table>
        </div>
        <div id="dig_reg_lb_footer">
            <div class="dig_reg_lb_cancel">Done</div>
        </div>
        <input type="hidden" value="<?php echo content_url('/plugins/digits/assets/images/', __FILE__); ?>"
               id="image_path">
    </div>
<?php }

add_action("after_dig_custom_section", "dig_ext_custom_section", 10, 2);


function dig_ext_add_new_btn()
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) return;
    ?>
    <div id="dig_open_reg_lb"><?php _e('REGISTRATION FORM LOGIC BUILDER', 'digadvlogic'); ?></div>
<?php }

add_action("dig_cf_add_new_btn", "dig_ext_add_new_btn");

function after_dig_ext_update_data($post)
{
    $dig_reg_lb = $post['dig_reg_lb'];
    update_option('dig_reg_lb_data', $dig_reg_lb);
}

add_action("after_dig_update_data", "after_dig_ext_update_data", 10, 1);

function dig_ext_register_form()
{
    $reg_lb = stripslashes(get_option("dig_reg_lb_data"));
    if (!$reg_lb) $reg_lb = "{}"; ?>
    <input type="hidden" id="dig_reg_lb" name="dig_reg_lb"
           value='<?php echo $reg_lb; ?>'/>

    <?php
}

add_action("register_form", "dig_ext_register_form");


register_activation_hook(__FILE__, 'digadvlogc_activate');

function digadvlogc_activate()
{
    if (!wp_next_scheduled('digadvlogc_purge_event')) {
        wp_schedule_event(time(), 'daily', 'digadvlogc_purge_event');
    }
}

add_action('digadvlogc_purge_event', 'digadvlogc_purge');

function digadvlogc_purge()
{
    $days = 2;

    $wp_upload_dir = wp_get_upload_dir();
    $path = $wp_upload_dir['basedir'] . '/digits/tmp/';
    if ($handle = opendir($path)) {

        while (false !== ($file = readdir($handle))) {
            if (is_file($path . $file)) {
                if (filemtime($path . $file) < (time() - ($days * 24 * 60 * 60))) {
                    unlink($path . $file);
                }
            }
        }
    }
}

register_deactivation_hook(__FILE__, 'digadvlogc_deactivate');

function digadvlogc_deactivate()
{
    wp_clear_scheduled_hook('digadvlogc_purge_event');
}
