<?php

/*
 * Plugin Name: DIGITS: Email Filter Addon
 * Description: Want to stop emails from some specific domain or stop spam? With this filter out the domains you don’t want or whitelist some.
 * Version: 1.4
 * Plugin URI: https://digits.unitedover.com/addons
 * Author URI: https://www.unitedover.com/
 * Author: UnitedOver
 * Text Domain: digfilter
 * Requires PHP: 5.5
 * Domain Path: /languages
 */

if (!defined('ABSPATH')) {
    exit;
}


require dirname(__FILE__) . '/Puc/v4p6/Factory.php';
require dirname(__FILE__) . '/Puc/v4/Factory.php';
require dirname(__FILE__) . '/Puc/v4p6/Autoloader.php';
new Puc_v4p6_Autoloader();

foreach (
    array(
        'Plugin_UpdateChecker' => 'Puc_v4p6_Plugin_UpdateChecker',
        'Vcs_PluginUpdateChecker' => 'Puc_v4p6_Vcs_PluginUpdateChecker',
    )
    as $pucGeneralClass => $pucVersionedClass
) {
    Puc_v4_Factory::addVersion($pucGeneralClass, $pucVersionedClass, '4.6');

    Puc_v4p6_Factory::addVersion($pucGeneralClass, $pucVersionedClass, '4.6');
}

function digits_update_emailfil_settings()
{
    if (isset($_POST['dig_block_disposable_email'])) {
        $settings = array();

        $settings['block_disposable_email'] = sanitize_text_field($_POST['dig_block_disposable_email']);

        $settings['whitelisteddomains'] = dig_emailiflt_sanitize_to_lower($_POST['digemailfil_whitelisteddomains']);


        $settings['blacklisteddomains'] = dig_emailiflt_sanitize_to_lower($_POST['digemailfil_blacklisteddomains']);


        update_option('dig_emailfilt_options', $settings);
    }
}

add_action('digits_save_settings_data', 'digits_update_emailfil_settings');


function digits_addon_digfilter()
{
    return 'digfilter';
}

function dig_show_emailfil($active_tab)
{
    ?>
    <div data-tab="digfiltertab"
         class="dig_admin_in_pt digfiltertab digtabview <?php echo $active_tab == digits_addon_digfilter() ? 'digcurrentactive' : '" style="display:none;'; ?>">
        <?php digad_show_emailfilt_settings(); ?>
    </div>

    <?php

}

add_action('digits_settings_page', 'dig_show_emailfil');


function digad_show_emailfilt_values()
{
    $digemailfit_default = array(
        'block_disposable_email' => 1,
        'whitelisteddomains' => array(),
        'blacklisteddomains' => array(),
    );
    return get_option('dig_emailfilt_options', $digemailfit_default);

}

function digad_show_emailfilt_settings()
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) return;

    $digemailfit_options = digad_show_emailfilt_values();
    $dig_block_disposable_email = $digemailfit_options['block_disposable_email'];

    $whitelisteddomains = $digemailfit_options['whitelisteddomains'];

    $blacklisteddomains = $digemailfit_options['blacklisteddomains'];

    ?>

    <div class="dig_admin_head"><span><?php _e('Email Filter', 'digfilter'); ?></span></div>

    <table class="form-table">

        <tr>
            <th scope="row" style="vertical-align:top;"><label
                        for="digemailfil_whitelisteddomains"><?php _e('Whitelisted Domains', 'digfilter'); ?></label>
            </th>
            <td>

                <select name="digemailfil_whitelisteddomains[]" class="dig_multiselect_email_dynamic_enable dig_ignore_select2"
                        multiple="multiple">
                    <?php

                    foreach ($whitelisteddomains as $whitelisteddomain) {
                        echo '<option value="' . $whitelisteddomain . '" selected>' . $whitelisteddomain . '</option>';
                    }

                    ?>
                </select><br/>

            </td>
        </tr>

        <tr>
            <th scope="row" style="vertical-align:top;"><label
                        for="digemailfil_blacklisteddomains"><?php _e('Blacklisted Domains', 'digfilter'); ?></label>
            </th>
            <td>

                <select name="digemailfil_blacklisteddomains[]" class="dig_multiselect_email_dynamic_enable dig_ignore_select2"
                        multiple="multiple">
                    <?php

                    foreach ($blacklisteddomains as $blacklisteddomain) {
                        echo '<option value="' . $blacklisteddomain . '" selected>' . $blacklisteddomain . '</option>';
                    }

                    ?>
                </select><br/>

            </td>
        </tr>

        <tr>
            <th scope="row"><label
                        for="dig_block_disposable_email"><?php _e('Block Disposable Email Registration', 'digfilter'); ?>
                </label></th>
            <td>
                <select name="dig_block_disposable_email" id="dig_block_disposable_email">
                    <option value="1" <?php if ($dig_block_disposable_email == 1) {
                        echo 'selected=selected';
                    } ?> ><?php _e('Yes', 'digfilter'); ?></option>
                    <option value="0" <?php if ($dig_block_disposable_email == 0) {
                        echo 'selected=selected';
                    } ?> ><?php _e('No', 'digfilter'); ?></option>
                </select>
            </td>
        </tr>

    </table>

    <script>
        jQuery(document).ready(function () {
            jQuery(".dig_multiselect_email_dynamic_enable").select2({
                tags: true,
                tokenSeparators: [',', ' '],
                createTag: function (params) {
                    if (params.term.indexOf('.') === -1) {
                        // Return null to disable tag creation
                        return null;
                    }
                    return {
                        id: params.term,
                        text: params.term
                    }
                }
            });
        })
    </script>
    <?php
}

$digfilterUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
    'https://bridge.unitedover.com/updates/changelog/addons.php?addon=digfilter',
    __FILE__,
    'digfilter'
);

$digfilterUpdateChecker->addQueryArgFilter('digfilter_filter_update_checks');
function digfilter_filter_update_checks($queryArgs)
{

    $queryArgs['license_key'] = get_site_option('dig_purchasecode');

    $queryArgs['request_site'] = network_home_url();

    $queryArgs['license_type'] = get_site_option('dig_license_type', 1);

    $plugin_data = get_plugin_data(__FILE__);
    $plugin_version = $plugin_data['Version'];

    $queryArgs['version'] = $plugin_version;


    return $queryArgs;
}


function digfilter_addon($list)
{
    $list[] = 'digfilter';
    return $list;
}

add_filter('digits_addon', 'digfilter_addon');


function dig_filter_email($validation_error, $email)
{

    $digpc = get_site_option('dig_purchasecode');
    if (empty($email) || empty($digpc)) return $validation_error;
    $digemailfit_options = digad_show_emailfilt_values();


    $whitelisteddomains = $digemailfit_options['whitelisteddomains'];
    $dig_block_disposable_email = $digemailfit_options['block_disposable_email'];


    $blacklisteddomains = $digemailfit_options['blacklisteddomains'];


    $domain = strtolower(substr(strrchr($email, "@"), 1));

    $error_message = __("We don't allow email addresses from this provider", "digfilter");
    if (!empty($whitelisteddomains) && count($whitelisteddomains) > 0) {

        if (!in_array($domain, $whitelisteddomains)) {
            $validation_error->add("Mail", $error_message);
            return $validation_error;
        }
    }


    if ($dig_block_disposable_email == 1) {
        $list_loc = plugin_dir_path(__FILE__) . '/assets/list.txt';
        $list_json = file_get_contents($list_loc);
        $list = explode(PHP_EOL, $list_json);

        if (in_array($domain, $list)) {
            $validation_error->add("Mail", $error_message);
            return $validation_error;
        }
    }

    if (!empty($blacklisteddomains) && count($blacklisteddomains) > 0) {

        if (in_array($domain, $blacklisteddomains)) {
            $validation_error->add("Mail", $error_message);
            return $validation_error;
        }
    }


    return $validation_error;
}

add_filter('digits_validate_email', 'dig_filter_email', 10, 2);


function dig_emailiflt_sanitize_to_lower($input)
{

    // Initialize the new array that will hold the sanitize values
    $new_input = array();

    // Loop through the input and sanitize each of the values
    foreach ($input as $key => $val) {
        $new_input[$key] = strtolower(sanitize_text_field($val));
    }

    return $new_input;

}

?>