<?php

/*
 * Plugin Name: DIGITS: REST API Addon
 * Description: This Digits addon allows you to use REST API calls for its functions.
 * Version: 2.1
 * Plugin URI: https://digits.unitedover.com/addons
 * Author URI: https://www.unitedover.com/
 * Author: UnitedOver
 * Text Domain: digrestapi
 * Requires PHP: 5.5
 * Domain Path: /languages
 */

if (!defined('ABSPATH')) {
    exit;
}

require dirname(__FILE__) . '/functions.php';
include dirname(__FILE__) . '/jwt/JWT.php';
include dirname(__FILE__) . '/jwt/BeforeValidException.php';
include dirname(__FILE__) . '/jwt/ExpiredException.php';
include dirname(__FILE__) . '/jwt/SignatureInvalidException.php';

require dirname(__FILE__) . '/Puc/v4p6/Factory.php';
require dirname(__FILE__) . '/Puc/v4/Factory.php';
require dirname(__FILE__) . '/Puc/v4p6/Autoloader.php';
new Puc_v4p6_Autoloader();

foreach (
    array(
        'Plugin_UpdateChecker' => 'Puc_v4p6_Plugin_UpdateChecker',
        'Vcs_PluginUpdateChecker' => 'Puc_v4p6_Vcs_PluginUpdateChecker',
    )
    as $pucGeneralClass => $pucVersionedClass
) {
    Puc_v4_Factory::addVersion($pucGeneralClass, $pucVersionedClass, '4.6');

    Puc_v4p6_Factory::addVersion($pucGeneralClass, $pucVersionedClass, '4.6');
}


function digits_update_restapi_settings()
{
    if (isset($_POST['dig_enable_rest_api'])) {
        $dig_enable_rest_api = sanitize_text_field($_POST['dig_enable_rest_api']);
        update_option('dig_enable_rest_api', $dig_enable_rest_api);

        $dig_token_life_rest_api = sanitize_text_field($_POST['dig_token_life_rest_api']);
        if (!is_numeric($dig_token_life_rest_api) || $dig_token_life_rest_api == 0) {
            $dig_token_life_rest_api = 14;
        }
        update_option('dig_token_life_rest_api', $dig_token_life_rest_api);

    }
}

add_action('digits_save_settings_data', 'digits_update_restapi_settings');


function digits_addon_digrestapi()
{
    return 'digrestapi';
}


function dig_show_restapi($active_tab)
{
    ?>
    <div data-tab="digrestapitab"
         class="dig_admin_in_pt digrestapitab digtabview <?php echo $active_tab == digits_addon_digrestapi() ? 'digcurrentactive' : '" style="display:none;'; ?>">
        <?php digad_show_restapi_settings(); ?>
    </div>

    <?php

}

add_action('digits_settings_page', 'dig_show_restapi');


function digad_show_restapi_settings()
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) {
        return;
    }

    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);
    $dig_token_life_rest_api = get_option('dig_token_life_rest_api', 14);

    ?>

    <div class="dig_admin_head"><span><?php _e('REST API', 'digrestapi'); ?></span></div>

    <table class="form-table">
        <tr>
            <th scope="row"><label
                        for="dig_enable_rest_api"><?php _e('Enable REST API', 'digrestapi'); ?>
                </label></th>
            <td>
                <select name="dig_enable_rest_api" id="dig_enable_rest_api">
                    <option value="1" <?php if ($dig_enable_rest_api == 1) {
                        echo 'selected=selected';
                    } ?> ><?php _e('Yes', 'digrestapi'); ?></option>
                    <option value="0" <?php if ($dig_enable_rest_api == 0) {
                        echo 'selected=selected';
                    } ?> ><?php _e('No', 'digrestapi'); ?></option>
                </select>
            </td>
        </tr>
        <tr>
            <th scope="row"><label
                        for="dig_token_life_rest_api"><?php _e('Access Token Validity (in days)', 'digrestapi'); ?>
                </label></th>
            <td>
                <input type="number" name="dig_token_life_rest_api" id="dig_token_life_rest_api"
                       value="<?php echo $dig_token_life_rest_api; ?>"/>
            </td>
        </tr>

    </table>

    <table class="form-table">
        <tbody>
        <tr>
            <td>
                <button href="https://help.unitedover.com/digits/kb/rest-api/?utm_source=digits-wp-settings&utm_medium=restapi-settings"
                        class="button dig_request_server_addition" type="button">
                    <?php _e('View Documentation', 'digits'); ?>
                </button>
            </td>
        </tr>
        </tbody>
    </table>


    <?php
}

$digrestapiUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
    'https://bridge.unitedover.com/updates/changelog/addons.php?addon=digrestapi',
    __FILE__,
    'digrestapi'
);

$digrestapiUpdateChecker->addQueryArgFilter('digrestapi_filter_update_checks');
function digrestapi_filter_update_checks($queryArgs)
{


    $queryArgs['license_key'] = get_site_option('dig_purchasecode');


    $queryArgs['request_site'] = network_home_url();

    $queryArgs['license_type'] = get_site_option('dig_license_type', 1);

    $plugin_data = get_plugin_data(__FILE__);
    $plugin_version = $plugin_data['Version'];

    $queryArgs['version'] = $plugin_version;


    return $queryArgs;
}


function digrestapi_addon($list)
{
    $list[] = 'digrestapi';

    return $list;
}

add_filter('digits_addon', 'digrestapi_addon');


add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/login_user/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_login_user',
        'permission_callback' => '__return_true'
    ));
});

add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/create_user/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_create_user',
        'permission_callback' => '__return_true'
    ));
});
add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/recovery/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_forgot_pass',
        'permission_callback' => '__return_true'
    ));
});
add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/send_otp/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_send_otp',
        'permission_callback' => '__return_true'
    ));
});
add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/resend_otp/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_resend_otp',
        'permission_callback' => '__return_true'
    ));
});

add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/verify_otp/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_verify_otp',
        'permission_callback' => '__return_true'
    ));
});

add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/logout/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_logout',
        'permission_callback' => '__return_true'
    ));
});

add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/one_click/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_one_click',
        'permission_callback' => '__return_true'
    ));
});

add_action('rest_api_init', function () {
    register_rest_route('digits/v1', '/update_phone/', array(
        'methods' => 'GET,POST',
        'callback' => 'digrest_update_phone',
        'permission_callback' => '__return_true'
    ));
});


function digrest_create_user()
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);
    if ($dig_enable_rest_api == 0) {
        return;
    }
    digrest_set_variables();

    $data = digits_create_user();
    if ($data['success'] === false) {
        wp_send_json($data);
    } else {
        $user_id = $data['data']['user_id'];
        digrest_create_token($user_id);
    }
    die();
}

function digrest_forgot_pass()
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);
    if ($dig_enable_rest_api == 0) {
        return;
    }

    digrest_set_variables();
    $data = digits_process_forgotpassword();

    if ($data['success'] === false) {
        wp_send_json($data);
    } else {
        if (isset($data['data']['show_msg'])) {
            wp_send_json($data);
        } else {
            digrest_create_token(get_current_user_id());
        }
    }
    die();
}

function digrest_send_otp()
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);
    if ($dig_enable_rest_api == 0) {
        return;
    }

    digrest_set_variables();

    $_REQUEST['csrf'] = wp_create_nonce('dig_form');
    $_POST['csrf'] = wp_create_nonce('dig_form');

    digits_check_mob();
}

function digrest_resend_otp()
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);
    if ($dig_enable_rest_api == 0) {
        return;
    }

    digrest_set_variables();

    $_REQUEST['csrf'] = wp_create_nonce('dig_form');
    $_POST['csrf'] = wp_create_nonce('dig_form');

    digits_resendotp();
}

function digrest_one_click()
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);
    if ($dig_enable_rest_api == 0) {
        return;
    }

    digrest_set_variables();

    $data = digoneclickls_process();

    if ($data['success'] === false) {
        wp_send_json($data);
    } else {
        if (isset($data['data']['show_msg'])) {
            wp_send_json($data);
        } else {
            digrest_create_token(get_current_user_id());
        }
    }
    die();


}

function digrest_set_variables()
{
    $digpc = get_site_option('dig_purchasecode');
    if (empty($digpc)) {
        return;
    }
    $_POST['digits'] = 1;

    if (isset($_REQUEST['type'])) {
        $type = $_REQUEST['type'];
        if ($type == 'login') {
            $_REQUEST['login'] = 1;
        }
        if ($type == 'register') {
            $_REQUEST['login'] = 2;
        } else if ($type == 'resetpass') {
            $_REQUEST['login'] = 3;
        } else if ($type == 'update') {
            $_REQUEST['login'] = 11;
        }
    }
    if (isset($_REQUEST['otp'])) {
        $_POST['dig_otp'] = $_REQUEST['otp'];
    }

    if (isset($_REQUEST['password'])) {
        $_POST['digits_password'] = $_REQUEST['password'];
        $_POST['digits_cpassword'] = $_REQUEST['password'];
    }


    if (isset($_POST['digits_reg_mobile'])) {
        $_POST['digits_reg_mail'] = $_POST['digits_reg_mobile'];
    }
    if (isset($_POST['digits_reg_email'])) {
        $_POST['dig_reg_mail'] = $_POST['digits_reg_email'];
    }
    if (isset($_POST['digits_reg_countrycode'])) {
        $_POST['digregcode'] = $_POST['digits_reg_countrycode'];
    }
    if (isset($_POST['user'])) {
        $_POST['forgotmail'] = $_POST['user'];
    }

    $_REQUEST['csrf'] = wp_create_nonce('crsf-otp');
    $_POST['csrf'] = wp_create_nonce('crsf-otp');

    $_POST['dig_nounce'] = wp_create_nonce('dig_form');
    $_POST['crsf-otp'] = wp_create_nonce('crsf-otp');

    $_REQUEST['json'] = 1;

}

function digrest_verify_otp()
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);
    if ($dig_enable_rest_api == 0) {
        return;
    }

    digrest_set_variables();

    $countrycode = sanitize_text_field($_REQUEST['countrycode']);

    if (dig_gatewayToUse($countrycode) == 1) {
        die();
    }

    $data = array();
    $mobileno = sanitize_mobile_field_dig($_REQUEST['mobileNo']);
    $del = false;
    $otp = sanitize_text_field($_REQUEST['otp']);
    if (verifyOTP($countrycode, $mobileno, $otp, $del)) {
        $data['code'] = 1;
        digit_send_json_status($data);
        die();
    } else {
        $data['code'] = 0;
        $data['message'] = __('Invalid OTP!', 'digits');
        digit_send_json_status($data);
        die();
    }
}


function digrest_update_phone()
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);

    if ($dig_enable_rest_api == 0) {
        return;
    }
    digrest_set_variables();

    if (!is_user_logged_in()) {
        wp_send_json_error(array('message' => __('Error! You need to be logged in to change number', 'digits')));
    } else {
        $mobile = sanitize_mobile_field_dig($_POST['mobileNo']);
        $countrycode = sanitize_text_field($_POST['countrycode']);
        $otp = isset($_POST['otp']) ? sanitize_text_field($_POST['otp']) : '';
        if (!is_numeric($mobile) || !is_numeric($countrycode)) {
            wp_send_json_error(array('message' => __('Please enter a valid Mobile Number', 'digits')));
        }
        if (empty($otp)) {
            wp_send_json_error(array('message' => __('Please enter a valid otp', 'digits')));
        }
        $csrf = sanitize_text_field($_POST['csrf']);

        $validateMob = dig_validateMobileNumber($countrycode, $mobile, $otp, $csrf, 11, $otp, false);

        if ($validateMob['success'] === false) {
            wp_send_json_error(array('message' => $validateMob['msg']));
        } else {
            digits_update_mobile(get_current_user_id(), $countrycode, $mobile);
            wp_send_json_success();
        }

    }
}


function digrest_login_user()
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);

    if ($dig_enable_rest_api == 0) {
        return;
    }

    digrest_set_variables();

    if (!isset($_POST['user']) && !isset($_POST['code']) && !isset($_POST['password'])) {
        wp_send_json_error(array('message' => __('Please enter valid credentials', 'digits')));
    }
    if (!is_user_logged_in()) {
        $code = isset($_POST['code']) ? sanitize_text_field($_POST['code']) : '';

        if (empty($code)) {
            $login = sanitize_mobile_field_dig($_POST['user']);
        } else {
            $login = -1;
        }
        if (!isset($_POST['password']) || empty($_POST['password'])) {
            $countrycode = isset($_POST['countrycode']) ? sanitize_text_field($_POST['countrycode']) : '-1';

            $csrf = sanitize_text_field($_POST['csrf']);

            $otp = isset($_POST['otp']) ? sanitize_text_field($_POST['otp']) : '';

            $validateMob = dig_validateMobileNumber($countrycode, $login, $otp, $csrf, 1, $code, false);

            if ($validateMob['success'] === false) {
                wp_send_json_error(array('message' => $validateMob['msg']));
            }

            $user = getUserFromPhone($validateMob['countrycode'] . $validateMob['mobile']);

        } else {
            if (is_numeric($login) && isset($_POST['countrycode'])) {
                $countrycode = sanitize_text_field($_POST['countrycode']);
                $temp_user = getUserFromPhone($countrycode . $login);
                if ($temp_user != null) {
                    $login = $temp_user->user_login;
                } else {
                    $login = sanitize_text_field($_POST['user']);
                }

            } else {
                $login = sanitize_text_field($_POST['user']);
            }
            $password = sanitize_text_field($_POST['password']);
            $user = wp_authenticate($login, $password);

        }

        if ($user == null || is_wp_error($user)) {
            if (is_wp_error($user)) {
                $invalid_message = $user->get_error_message();
            } else {
                $invalid_message = __("Please signup before logging in.", 'digits');
            }
            wp_send_json_error(array('message' => strip_tags($invalid_message)));

        } else {
            $user_id = $user->ID;
            digrest_create_token($user_id);
        }

    } else {
        if (is_user_logged_in()) {
            wp_send_json_error(array('message' => __('User is already logged in!', 'digits')));
        }
    }


}

if (!function_exists('digrest_create_token')) {
    function digrest_create_token($user_id)
    {
        if ($user_id == 0) {
            wp_send_json_error(array('message' => __('Error: Unexpected user', 'digits')));
        }

        $access_token = bin2hex(random_bytes(64));
        $refresh_token = bin2hex(random_bytes(64));
        global $wpdb;
        $table_name = $wpdb->prefix . "digits_digrest_token";

        $user_id = sanitize_text_field($user_id);
        $access_token = sanitize_text_field($access_token);
        $refresh_token = sanitize_text_field($refresh_token);
        $ip = sanitize_text_field(digrestapi_getip());
        $user_agent = sanitize_text_field($_SERVER['HTTP_USER_AGENT']);

        $access_token = apply_filters('digits_rest_token', $access_token, $user_id);

        $db = $wpdb->insert($table_name, array(
            'user_id' => $user_id,
            'access_token' => $access_token,
            'refresh_token' => '',
            'user_agent' => $user_agent,
            'ip' => $ip,
            'last_time' => date("Y-m-d H:i:s", strtotime("now")),
            'create_time' => date("Y-m-d H:i:s", strtotime("now")),
        ), array(
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
            )
        );

        if (!$db) {
            wp_send_json_error(array('message' => __('Error while creating access token', 'digits')));
        } else {
            nocache_headers();

            $key = digrest_key();

            $data = array(
                "user_id" => $user_id,
                "access_token" => $access_token,
                "token_type" => "bearer"

            );

            $data = apply_filters('digits_rest_token_data', $data, $user_id);

            wp_send_json_success($data);
        }
    }
}

function digrestapi_getAuthorizationHeader()
{
    $headers = null;
    if (isset($_SERVER['Authorization'])) {
        $headers = trim($_SERVER["Authorization"]);
    } else if (isset($_SERVER['HTTP_AUTHORIZATION'])) {
        $headers = trim($_SERVER["HTTP_AUTHORIZATION"]);
    } elseif (function_exists('apache_request_headers')) {
        $requestHeaders = apache_request_headers();
        $requestHeaders = array_combine(array_map('ucwords', array_keys($requestHeaders)), array_values($requestHeaders));
        if (isset($requestHeaders['Authorization'])) {
            $headers = trim($requestHeaders['Authorization']);
        }
    }

    return $headers;
}

function digrestapi_getBearerToken()
{
    $headers = digrestapi_getAuthorizationHeader();
    if (!empty($headers)) {
        if (preg_match('/Bearer\s(\S+)/', $headers, $matches)) {
            return $matches[1];
        }
    }

    return null;
}

function digrestapi_auth_handler($user)
{
    $dig_enable_rest_api = get_option('dig_enable_rest_api', 1);
    if ($dig_enable_rest_api == 0) {
        return $user;
    }

    $wp_json_basic_auth_error = null;
    if (!empty($user)) {
        return $user;
    }

    $token = digrestapi_getBearerToken();


    if (!$token || $token == null) {
        return $user;
    }


    $auth_token = sanitize_text_field($token);

    $check_token = digits_get_access_token_details($token);

    if ($check_token == null) {
        return $user;
    } else {

        $user_id = $check_token->user_id;

        $time = strtotime($check_token->last_time);
        $current = strtotime("now");


        $expiration = get_option('dig_token_life_rest_api', 14) * DAY_IN_SECONDS;

        if ($current - $time > $expiration) {
            digits_delete_access_token($auth_token);

            wp_send_json_error(array(
                'error' => 'token_expired',
                'message' => __('Your session has expired. Please login again', 'digits')
            ), 401);
            die();
        } else {
            global $wpdb;
            $table_name = $wpdb->prefix . "digits_digrest_token";

            $wpdb->query(
                $wpdb->prepare(
                    'UPDATE ' . $table_name . ' SET  
        last_time = %s WHERE access_token = %s ',
                    date("Y-m-d H:i:s", strtotime("now")), $auth_token
                )
            );
        }
    }

    return $user_id;
}

add_filter('determine_current_user', 'digrestapi_auth_handler');


function digrest_key()
{
    $key = get_option('digits_rest_key', -1);
    if ($key == -1) {
        $key = bin2hex(random_bytes(16));
        update_option('digits_rest_key', $key);
    }

    return $key;
}

function digrest_logout()
{
    if (is_user_logged_in()) {
        wp_destroy_current_session();
        wp_clear_auth_cookie();
        wp_set_current_user( 0 );
    }
    $token = digrestapi_getBearerToken();
    if (!$token || $token == null) {
        wp_send_json_error();
    }
    $token = sanitize_textarea_field($token);

    $result = digits_delete_access_token($token);

    if (!$result) {
        wp_send_json_error();
    } else {
        wp_send_json_success();
    }
}

function digrest_delete_token($email, $user, $blogname)
{
    $user_id = sanitize_textarea_field($user->ID);

    $delete = digits_delete_user_access_tokens($user_id);


    return $email;
}

add_filter('wp_password_change_notification_email', 'digrest_delete_token', 10, 3);

function digrest_password_reset($user, $new_pass)
{
    digrest_delete_token(null, $user, null);
}

add_action('password_reset', 'digrest_password_reset', 10, 2);


register_activation_hook(__FILE__, 'digrest_activate');

function digrest_activate()
{
    global $wpdb;
    $tb = $wpdb->prefix . 'digits_digrest_token';
    if ($wpdb->get_var("SHOW TABLES LIKE '$tb'") != $tb) {
        $charset_collate = $wpdb->get_charset_collate();
        $sql = "CREATE TABLE $tb (
	id BIGINT UNSIGNED NOT null auto_increment,
		          user_id BIGINT UNSIGNED NOT null,
		          access_token TEXT NOT null,
		          refresh_token TEXT NOT null,
		          user_agent TEXT NOT null,
		          ip TEXT NOT null,
		          last_time datetime DEFAULT CURRENT_TIMESTAMP NOT null,
		          create_time datetime DEFAULT CURRENT_TIMESTAMP NOT null,
		          PRIMARY KEY( id )
	            ) $charset_collate;";
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta(array($sql));

        digrest_key();
        update_option('digits_rest_key', $key);
    }

}

function digrestapi_getip()
{

    foreach (
        array(
            'HTTP_CLIENT_IP',
            'HTTP_X_FORWARDED_FOR',
            'HTTP_X_FORWARDED',
            'HTTP_X_CLUSTER_CLIENT_IP',
            'HTTP_FORWARDED_FOR',
            'HTTP_FORWARDED',
            'REMOTE_ADDR'
        ) as $key
    ) {
        if (array_key_exists($key, $_SERVER) === true) {
            foreach (array_map('trim', explode(',', $_SERVER[$key])) as $ip) {
                if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE) !== false) {
                    return $ip;
                }
            }
        }
    }
}

?>