<?php


if (!defined('ABSPATH')) {
    exit;
}

/**
 * @returns array of
 * id - token id
 * user_id - WP User ID
 * access_token
 * ip - ip from which token was created
 * last_time - last time when token was used to authorize user
 * create_time - when token was created
 * */
function digits_get_user_access_tokens($user_id)
{
    if (empty($user_id) || !is_numeric($user_id)) {
        return null;
    }

    global $wpdb;
    $table_name = $wpdb->prefix . "digits_digrest_token";

    return $wpdb->get_results(
        $wpdb->prepare(
            'SELECT * FROM ' . $table_name . ' WHERE access_token = %s',
            $user_id
        )
    );
}

function digits_get_user_from_access_token($auth_token)
{
    $token_details = digits_get_access_token_details($auth_token);

    if ($token_details != null) {
        $user_id = $token_details->user_id;
        $user = get_user_by('ID', $user_id);

        return !$user ? null : $user;

    }
    return null;
}

function digits_get_access_token_details($auth_token)
{
    if (empty($auth_token)) {
        return null;
    }

    global $wpdb;
    $table_name = $wpdb->prefix . "digits_digrest_token";

    return $wpdb->get_row(
        $wpdb->prepare(
            'SELECT * FROM ' . $table_name . ' WHERE access_token = %s',
            $auth_token
        )
    );
}


function digits_delete_access_token($auth_token)
{
    global $wpdb;
    $table_name = $wpdb->prefix . "digits_digrest_token";

    return $wpdb->delete($table_name, array(
        'access_token' => $auth_token,
    ), array(
            '%s'
        )
    );

}

function digits_delete_user_access_tokens($user_id)
{
    if (empty($user_id) || !is_numeric($user_id)) return null;

    global $wpdb;
    $table_name = $wpdb->prefix . "digits_digrest_token";

    return $wpdb->delete($table_name, array(
        'user_id' => $user_id,
    ), array(
            '%s'
        )
    );
}